pro read_lunar_hdf5, filename, status, lunar

;Disclaimers
;No Warranty: THE SUBJECT SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY OF
;ANY KIND, EITHER EXPRESSED, IMPLIED, OR STATUTORY, INCLUDING, BUT NOT LIMITED
;TO, ANY WARRANTY THAT THE SUBJECT SOFTWARE WILL CONFORM TO SPECIFICATIONS,
;ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
;OR FREEDOM FROM INFRINGEMENT, ANY WARRANTY THAT THE SUBJECT SOFTWARE WILL BE
;ERROR FREE, OR ANY WARRANTY THAT DOCUMENTATION, IF PROVIDED, WILL CONFORM TO
;THE SUBJECT SOFTWARE. THIS AGREEMENT DOES NOT, IN ANY MANNER, CONSTITUTE AN
;ENDORSEMENT BY GOVERNMENT AGENCY OR ANY PRIOR RECIPIENT OF ANY RESULTS,
;RESULTING DESIGNS, HARDWARE, SOFTWARE PRODUCTS OR ANY OTHER APPLICATIONS
;RESULTING FROM USE OF THE SUBJECT SOFTWARE.  FURTHER, GOVERNMENT AGENCY
;DISCLAIMS ALL WARRANTIES AND LIABILITIES REGARDING THIRD-PARTY SOFTWARE, IF
;PRESENT IN THE ORIGINAL SOFTWARE, AND DISTRIBUTES IT "AS IS." 
 
;Waiver and Indemnity:  RECIPIENT AGREES TO WAIVE ANY AND ALL CLAIMS AGAINST
;THE UNITED STATES GOVERNMENT, ITS CONTRACTORS AND SUBCONTRACTORS, AS WELL AS
;ANY PRIOR RECIPIENT.  IF RECIPIENT'S USE OF THE SUBJECT SOFTWARE RESULTS IN
;ANY LIABILITIES, DEMANDS, DAMAGES, EXPENSES OR LOSSES ARISING FROM SUCH USE,
;INCLUDING ANY DAMAGES FROM PRODUCTS BASED ON, OR RESULTING FROM, RECIPIENT'S
;USE OF THE SUBJECT SOFTWARE, RECIPIENT SHALL INDEMNIFY AND HOLD HARMLESS THE
;UNITED STATES GOVERNMENT, ITS CONTRACTORS AND SUBCONTRACTORS, AS WELL AS ANY
;PRIOR RECIPIENT, TO THE EXTENT PERMITTED BY LAW.  RECIPIENT'S SOLE REMEDY FOR
;ANY SUCH MATTER SHALL BE THE IMMEDIATE, UNILATERAL TERMINATION OF THIS
;AGREEMENT.

;  Purpose:
;  This provides access to the data stored in the lunar species HDF file
;  specifically those HDF version 5 files
;  
;  Author:
;  Michael Heitz <michael.l.heitz@nasa.gov>
;  Carter Hulsey
;  Robbie Manion
;  Marsha Larosee
;  David Huber
;  Lawrence Selvy
;
;  Input Variables:
;  filename - name of the lunar species HDF version 5 file (g3a.lsp.*)
;
;  Output Variables:
;  status - status of the read 
;  lunar - data structure containing the lunar species information


status = 'OK'

;Find Version from file name and if greater than or equal to 5.1
;Use the updated Reader
splt_file = STRSPLIT(filename,'v',/EXTRACT)
vers = FLOAT(splt_file[-1])

IF vers GE 5.2 THEN BEGIN
  print, 'Reader does not support version 5.2 HDF files'
  print, 'Exiting...'
  exit

ENDIF ELSE IF vers EQ 5.1 THEN BEGIN
	;  Create the structure for read from binary file of the data needed
	;  to dynamically size data arrays.

	lunar_size = {event_id:0L, $
	          Date:0L, $
	          fraction_time:0., $
	          Latitude:0., $
	          Longitude:0., $
	          Time:0L, $
	          int_fill_value:0L, $
	          flt_fill_value:0., $
	          mission_id:0L, $
	          L0DO_Version:0., $
	          CCDVersion:0., $
	          L0_Version:0., $
	          Software_Version:0., $
	          Dataproduct_Version:0., $
	          Spectroscopic_DataBase_Version:0., $
	          GRAM95_Version:0., $
	          Met_Version:0., $
	          Lun_Model_Ver:0., $
	          Lun_Albedo_Ver:0., $
	          bin_height:0., $
	          num_alt_bins:0L, $
	          num_press_grid:0L, $
	          num_grnd_trk:0L }

	!QUIET = 1

        ;  Now we open the file
        fileID = H5F_OPEN(filename)

        nMainGroups = h5g_get_nmembers(fileID,"/")

        ;  Now we select the first group and start parsing the data from the HDF5 file
        g0Name = h5g_get_member_name(fileID,"/",0)
        g0Path = "/" + g0Name
        g01Name = h5g_get_member_name(fileID,g0Path,0)
        g01Path = g0Path + "/" + g01Name
        dset_id = h5d_open(fileID,g01Path)
        data = h5d_read(dset_id)
        lunar_size.event_id = data.(0)
        lunar_size.fraction_time = data.(1)
        lunar_size.Date = data.(2)
        lunar_size.Latitude = data.(3)
        lunar_size.Longitude = data.(4)
        lunar_size.Time = data.(5)
        lunar_size.int_fill_value = data.(6)
        lunar_size.flt_fill_value = data.(7)
        lunar_size.mission_id = data.(8)
        h5d_close,dset_id

        g1Name = h5g_get_member_Name(fileID,"/",1)
        g1Path = "/" + g1Name
        g11Name = h5g_get_member_Name(fileID,g1Path,0)
        g11Path = g1Name + "/" + g11Name
        dset_id = h5d_open(fileID,g11Path)
        data = h5d_read(dset_id)
        lunar_size.L0DO_Version = data.(0)
        lunar_size.CCDVersion = data.(1)
        lunar_size.L0_Version = data.(2)
        lunar_size.Spectroscopic_DataBase_Version = data.(3)
        lunar_size.GRAM95_Version = data.(4)
        lunar_size.Met_Version = data.(5)
        lunar_size.Software_Version = data.(6)
        lunar_size.Dataproduct_Version = data.(7)
        lunar_size.Lun_Model_Ver = data.(8)
        lunar_size.Lun_Albedo_Ver = data.(9)
        h5d_close,dset_id

        g2Name = h5g_get_member_Name(fileID,"/",2)
        g2Path = "/" + g2Name
        g21Name = h5g_get_member_Name(fileID,g2Path,0)
        g21Path = g2Path + "/" + g21Name
        dset_id = h5d_open(fileID,g21Path)
        data = h5d_read(dset_id)
        lunar_size.bin_height = data.(0)
        lunar_size.num_alt_bins = data.(1)
        lunar_size.num_grnd_trk = data.(2)
        lunar_size.num_press_grid = data.(3)
        h5d_close,dset_id

	;Creating Lunar Structure 
	lunar = { Event_ID:0L, $
	     Date:0L, $
	     Year_Fraction:0., $
	     Latitude:0., $
	     Longitude:0., $
	     Time:0L, $
	     int_fill_value:0L, $
	     flt_fill_value:0., $
	     mission_id:0L, $
	     L0DO_Version:0., $
	     CCDTable_Version:0L, $
	     L0_Version:0., $
	     Software_Version:0., $
	     Dataproduct_Version:0., $
	     Spectroscopic_DataBase_Version:0., $
	     GRAM95_Version:0., $
	     Met_Version:0., $
	     Lun_Model_Ver:0., $
	     Lun_Albedo_Ver:0., $
	     bin_height:0., $
	     num_alt_bins:0L, $
	     num_press_grid:0L, $
	     num_grnd_trk:0L, $
	     sc_evt_type:0L, $
	     gnd_evt_type:0L, $
	     BetaAngle_Lunar:0., $
	     LunarPhase:0., $
	     ZenithAngle:0., $
	     aurora_flag:0L, $
	     Ephemeris_Source:0L, $
	     gt_date:lonarr(lunar_size.num_grnd_trk), $
	     gt_time:lonarr(lunar_size.num_grnd_trk), $
	     gt_latitude:fltarr(lunar_size.num_grnd_trk), $
	     gt_longitude:fltarr(lunar_size.num_grnd_trk), $
	     gt_ray_dir:fltarr(lunar_size.num_grnd_trk), $
	     Space_Craft_Lat:fltarr(lunar_size.num_grnd_trk), $
	     Space_Craft_Lon:fltarr(lunar_size.num_grnd_trk), $
	     Space_Craft_Alt:fltarr(lunar_size.num_grnd_trk), $
	     Altitude:fltarr(lunar_size.num_alt_bins), $
	     Geopotential_Alt:fltarr(lunar_size.num_alt_bins), $
	     Temperature:fltarr(lunar_size.num_alt_bins), $
	     Temperature_uncert:fltarr(lunar_size.num_alt_bins), $
	     Pressure:fltarr(lunar_size.num_alt_bins), $
	     Pressure_uncert:fltarr(lunar_size.num_alt_bins), $
	     Neutral_Density:fltarr(lunar_size.num_alt_bins), $
	     Neutral_Density_uncert:fltarr(lunar_size.num_alt_bins), $
	     Temp_Pressure_Source:lonarr(lunar_size.num_alt_bins), $
	     trop_temp:0., $
	     trop_alt:0., $
	     trop_press:0., $
	     met_pressure:fltarr(lunar_size.num_press_grid), $
	     met_temp:fltarr(lunar_size.num_press_grid), $
	     met_temp_unc:fltarr(lunar_size.num_press_grid), $
	     met_altitude:fltarr(lunar_size.num_press_grid),$
	     met_source: 0L, $
	     CCD_Temperature:0., $
	     Spectrometer_Zenith_Temperature:0., $
	     CCD_Temperature_minus_TEC:0., $
	     Ephemeris_Quality:0L, $
	     QAFlag:0L, $
	     QAFlag_Altitude:fltarr(lunar_size.num_alt_bins), $
	     SpecCalStretch:0., $
	     SpecCalShift:0., $
	     AbandAltRegQA:lonarr(lunar_size.num_alt_bins), $
	     AbandAltRegOffset:0., $
	     Ozone:fltarr(lunar_size.num_alt_bins), $
	     Ozone_uncert:fltarr(lunar_size.num_alt_bins), $
	     OzoneQA:lonarr(lunar_size.num_alt_bins), $
	     NO2:fltarr(lunar_size.num_alt_bins), $
	     NO2_uncert:fltarr(lunar_size.num_alt_bins), $
	     NO2QA:lonarr(lunar_size.num_alt_bins), $
	     NO3:fltarr(lunar_size.num_alt_bins), $
	     NO3_uncert:fltarr(lunar_size.num_alt_bins), $
	     NO3QA:lonarr(lunar_size.num_alt_bins), $
	     OClO:fltarr(lunar_size.num_alt_bins), $
	     OClO_uncert:fltarr(lunar_size.num_alt_bins), $
	     OClOQA:lonarr(lunar_size.num_alt_bins) }

	;  Now we read in the data from the lunar_size array
        lunar.event_id = lunar_size.event_id
	lunar.Date = lunar_size.Date
        lunar.Year_Fraction = lunar_size.Fraction_time
        lunar.Latitude = lunar_size.Latitude
	lunar.Longitude = lunar_size.Longitude
	lunar.Time = lunar_size.Time
        lunar.int_fill_value = lunar_size.int_fill_value
        lunar.flt_fill_value = lunar_size.flt_fill_value
        lunar.mission_id = lunar_size.mission_id
        lunar.L0DO_Version = lunar_size.L0DO_Version
	lunar.CCDTable_Version = lunar_size.CCDVersion
        lunar.L0_Version = lunar_size.L0_Version
        lunar.Software_Version = lunar_size.Software_Version
        lunar.Dataproduct_Version = lunar_size.Dataproduct_Version
        lunar.Spectroscopic_DataBase_Version = lunar_size.Spectroscopic_DataBase_Version
        lunar.GRAM95_Version = lunar_size.GRAM95_Version
        lunar.Met_Version = lunar_size.Met_Version
        lunar.bin_height = lunar_size.bin_height
        lunar.num_alt_bins = lunar_size.num_alt_bins
        lunar.num_grnd_trk = lunar_size.num_grnd_trk
        lunar.num_press_grid = lunar_size.num_press_grid
        lunar.Lun_Model_Ver = lunar_size.Lun_Model_Ver
        lunar.Lun_Albedo_Ver = lunar_size.Lun_Albedo_Ver

 	;  Now that the structure exists and has been filled with the previously
        ;  read data, we now fill the rest of the structure
	;Section 4 Event Identification
        g3Name = h5g_get_member_Name(fileID,"/",3)
        g3Path = "/" + g3Name
        dsName = h5g_get_member_Name(fileID,g3Path,0)
        dsPath = g3Path + "/" + dsName
        dset_id = h5d_open(fileID,dsPath)
        data = h5d_read(dset_id)
        lunar.sc_evt_type = data.(0)
        lunar.gnd_evt_type = data.(1)
        lunar.BetaAngle_Lunar = data.(2)
        lunar.LunarPhase = data.(3)
        lunar.ZenithAngle = data.(4)
        lunar.aurora_flag = data.(5)
        lunar.Ephemeris_Source = data.(6)
        h5d_close,dset_id

        g31Name = h5g_get_member_Name(fileID,g3Path,1)
        g31Path = g3Path + "/" + g31Name
        dsName = h5g_get_member_Name(fileID,g31Path,0)
        dsPath = g31Path + "/" + dsName
        dset_id = h5d_open(fileID,dsPath)
        data = h5d_read(dset_id)
        lunar.gt_date = data.(0)
        lunar.gt_time = data.(1)
        lunar.gt_latitude = data.(2)
        lunar.gt_longitude = data.(3)
        lunar.gt_ray_dir = data.(4)
        h5d_close,dset_id

        g32Name = h5g_get_member_Name(fileID,g3Path,2)
        g32Path = g3Path + "/" + g32Name
        dsName = h5g_get_member_Name(fileID,g32Path,0)
        dsPath = g32Path + "/" + dsName
        dset_id = h5d_open(fileID,dsPath)
        data = h5d_read(dset_id)
        lunar.Space_Craft_Lat= data.(0)
        lunar.Space_Craft_Lon = data.(1)
        lunar.Space_Craft_Alt = data.(2)
        h5d_close,dset_id

        g33Name = h5g_get_member_Name(fileID,g3Path,3)
        g33Path = g3Path + "/" + g33Name
        dsName = h5g_get_member_Name(fileID,g33Path,0)
        dsPath = g33Path + "/" + dsName
        dset_id = h5d_open(fileID,dsPath)
        data = h5d_read(dset_id)
        lunar.CCD_Temperature = data.(0)
        lunar.Spectrometer_Zenith_Temperature = data.(1)
        lunar.CCD_Temperature_minus_TEC= data.(2)
        lunar.Ephemeris_Quality = data.(3)
        lunar.SpecCalShift = data.(4)
        lunar.SpecCalStretch = data.(5)
        lunar.QAFlag = data.(6)
        h5d_close,dset_id

        g34Name = h5g_get_member_Name(fileID,g3Path,4)
        g34Path = g3Path + "/" + g34Name
        dsName = h5g_get_member_Name(fileID,g34Path,0)
        dsPath = g34Path + "/" + dsName
        dset_id = h5d_open(fileID,dsPath)
        data = h5d_read(dset_id)
        lunar.QAFlag_Altitude = data.(0)
        h5d_close,dset_id

	;  Now we read in group 5
        g4Name = h5g_get_member_Name(fileID,"/",4)
        g4Path = "/" + g4Name
        dsName = h5g_get_member_name(fileID,g4Path,0)
        dsPath = g4Path + "/" + dsName
        dset_id = h5d_open(fileID,dsPath)
        data = h5d_read(dset_id)
        lunar.trop_alt = data.(0)
        lunar.trop_temp = data.(1)
        lunar.trop_press = data.(2)
	lunar.met_source = data.(3)
	h5d_close,dset_id

        ;  Now we read group 5.1
        g5Name = h5g_get_member_Name(fileID,"/",5)
        g5Path = "/" + g5Name
        dsName = h5g_get_member_Name(fileID,g5Path,0)
        dsPath = g5Path + "/" + dsName
        dset_id = h5d_open(fileID,dsPath)
        data = h5d_read(dset_id)
        lunar.met_altitude = data.(0)
        lunar.met_temp = data.(1)
        lunar.met_temp_unc = data.(2)
        lunar.met_pressure = data.(3)
        h5d_close,dset_id

 	; Now we read group group 6 
        g6Name = h5g_get_member_Name(fileID,"/",6)
        g6Path = "/" + g6Name
        dsName = h5g_get_member_Name(fileID,g6Path,0)
        dsPath = g6Path + "/" + dsName
        dset_id = h5d_open(fileID,dsPath)
        data = h5d_read(dset_id)
        lunar.Altitude = data.(0)
        lunar.AbandAltRegQA = data.(1)
        lunar.Geopotential_Alt = data.(2)
        h5d_close,dset_id

        ;  Now we read group 6A
        g7Name = h5g_get_member_Name(fileID,"/",7)
        g7Path = "/" + g7Name
        dsName = h5g_get_member_Name(fileID,g7Path,0)
        dsPath = g7Path + "/" + dsName
        dset_id = h5d_open(fileID,dsPath)
        data = h5d_read(dset_id)
        lunar.AbandAltRegOffset = data.(0)
        h5d_close,dset_id

        ;  Now we read group 6.1
        g8N = h5g_get_member_Name(fileID,"/",8)
        g8P = "/" + g8N
        dsN = h5g_get_member_Name(fileID,g8P,0)
        dsP = g8P + "/" +dsN
        dset_id = h5d_open(fileID,dsP)
        data = h5d_read(dset_id)
        lunar.Temperature = data.(0)
        lunar.Temperature_uncert = data.(1)
        lunar.Pressure = data.(2)
        lunar.Pressure_uncert = data.(3)
        lunar.Neutral_Density = data.(4)
	lunar.Neutral_Density_uncert = data.(5)
	lunar.Temp_Pressure_Source = data.(6)
        h5d_close,dset_id

        ;  Now we read group 6.2
        g9N = h5g_get_member_Name(fileID,"/",9)
        g9P = "/" + g9N
        dsN = h5g_get_member_Name(fileID,g9P,0)
        dsP = g9P + "/" + dsN
        dset_id = h5d_open(fileID,dsP)
        data = h5d_read(dset_id)
        lunar.Ozone = data.(0)
        lunar.Ozone_uncert = data.(1)
        lunar.OzoneQA = data.(2)
        h5d_close,dset_id

        ;  Now read group 6.3
        g10N = h5g_get_member_Name(fileID,"/",10)
        g10P = "/" + g10N
        dsN = h5g_get_member_Name(fileID,g10P,0)
        dsP = g10P + "/" + dsN
        dset_id = h5d_open(fileID,dsP)
        data = h5d_read(dset_id)
        lunar.NO2 = data.(0)
        lunar.NO2_uncert = data.(1)
        lunar.NO2QA = data.(2)
        h5d_close,dset_id

	; Now read group 6.4
        g11N = h5g_get_member_Name(fileID,"/",11)
        g11P = "/" + g11N
        dsN = h5g_get_member_Name(fileID,g11P,0)
        dsP = g11P + "/" + dsN
        dset_id = h5d_open(fileID,dsP)
        data = h5d_read(dset_id)
        lunar.NO3 = data.(0)
        lunar.NO3_uncert = data.(1)
        lunar.NO3QA = data.(2)
        h5d_close,dset_id

	; Now read Group 6.5
	g12N = h5g_get_member_Name(fileID,"/",12)
        g12P = "/" + g12N
        dsN = h5g_get_member_Name(fileID,g12P,0)
        dsP = g12P + "/" + dsN
        dset_id = h5d_open(fileID,dsP)
        data = h5d_read(dset_id)
        lunar.OClO = data.(0)
        lunar.OClO_uncert = data.(1)
        lunar.OClOQA = data.(2)
        h5d_close,dset_id

        h5f_close,fileID
	H5_CLOSE

        ;Add Separate Flags for Event Qa BitFlag Entries, IDL 1/0 = True/False
        lunar_addtags={HexErrFlag:0L,ContWindowClosedFlag:0L,TimeQualFlag:0L,DMPExoFlag:0L,SpectralCalFlag:0L,$
           DMPAltFlag:lonarr(lunar_size.num_alt_bins)}

        lunar=create_struct(lunar,lunar_addtags)

        if ((lunar.QAFlag and 2L^0) ne 0) then lunar.HexErrFlag=1 else lunar.HexErrFlag=0
        if ((lunar.QAFlag and 2L^1) ne 0) then lunar.ContWindowClosedFlag=1 else lunar.ContWindowClosedFlag=0
        if ((lunar.QAFlag and 2L^2) ne 0) then lunar.TimeQualFlag=1 else lunar.TimeQualFlag=0
        if ((lunar.QAFlag and 2L^3) ne 0) then lunar.DMPExoFlag=1 else lunar.DMPExoFlag=0
        if ((lunar.QAFlag and 2L^4) ne 0) then lunar.SpectralCalFlag=1 else lunar.SpectralCalFlag=0

        ;Altitude Based Bit Flags
        for i = 0,lunar_size.num_alt_bins-1 do begin
          if ((lunar.QAFlag_Altitude(i) and 2L^0) ne 0) then lunar.DMPAltFlag(i)=1 else lunar.DMPAltFlag(i)=0
        endfor


ENDIF ELSE BEGIN
	status = 'ERROR'
	print, 'HDF5 files only available for v5.1 and later'
ENDELSE


end


