pro read_solar_hdf5, fileName, status, solar

;Disclaimers
;No Warranty: THE SUBJECT SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY OF
;ANY KIND, EITHER EXPRESSED, IMPLIED, OR STATUTORY, INCLUDING, BUT NOT LIMITED
;TO, ANY WARRANTY THAT THE SUBJECT SOFTWARE WILL CONFORM TO SPECIFICATIONS,
;ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
;OR FREEDOM FROM INFRINGEMENT, ANY WARRANTY THAT THE SUBJECT SOFTWARE WILL BE
;ERROR FREE, OR ANY WARRANTY THAT DOCUMENTATION, IF PROVIDED, WILL CONFORM TO
;THE SUBJECT SOFTWARE. THIS AGREEMENT DOES NOT, IN ANY MANNER, CONSTITUTE AN
;ENDORSEMENT BY GOVERNMENT AGENCY OR ANY PRIOR RECIPIENT OF ANY RESULTS,
;RESULTING DESIGNS, HARDWARE, SOFTWARE PRODUCTS OR ANY OTHER APPLICATIONS
;RESULTING FROM USE OF THE SUBJECT SOFTWARE.  FURTHER, GOVERNMENT AGENCY
;DISCLAIMS ALL WARRANTIES AND LIABILITIES REGARDING THIRD-PARTY SOFTWARE, IF
;PRESENT IN THE ORIGINAL SOFTWARE, AND DISTRIBUTES IT "AS IS." 
 
;Waiver and Indemnity:  RECIPIENT AGREES TO WAIVE ANY AND ALL CLAIMS AGAINST
;THE UNITED STATES GOVERNMENT, ITS CONTRACTORS AND SUBCONTRACTORS, AS WELL AS
;ANY PRIOR RECIPIENT.  IF RECIPIENT'S USE OF THE SUBJECT SOFTWARE RESULTS IN
;ANY LIABILITIES, DEMANDS, DAMAGES, EXPENSES OR LOSSES ARISING FROM SUCH USE,
;INCLUDING ANY DAMAGES FROM PRODUCTS BASED ON, OR RESULTING FROM, RECIPIENT'S
;USE OF THE SUBJECT SOFTWARE, RECIPIENT SHALL INDEMNIFY AND HOLD HARMLESS THE
;UNITED STATES GOVERNMENT, ITS CONTRACTORS AND SUBCONTRACTORS, AS WELL AS ANY
;PRIOR RECIPIENT, TO THE EXTENT PERMITTED BY LAW.  RECIPIENT'S SOLE REMEDY FOR
;ANY SUCH MATTER SHALL BE THE IMMEDIATE, UNILATERAL TERMINATION OF THIS
;AGREEMENT.

;  Purpose:
;  This provides the data structures required to read in the solar SAGE III
;  event species data from HDF version 5 files
;
;  Authors:
;  Michael Heitz <michael.l.heitz@nasa.gov>
;  Carter Hulsey
;  Robbie Manion
;  Marsha Larosee
;  David Huber
;  Lawrence Selvy
;
;  Input Variables:
;  fileName - Name of the HDF5 file to be read in (g3a.ssp.*)
;
;  Output Variables:
;  status - Status variable of the read

status = 'OK'

save_solar_reader_version = 0

;Find Version from file name and if greater than or equal to 5.1
;Use the updated Reader
splt_file = STRSPLIT(filename,'v',/EXTRACT)
vers = FLOAT(splt_file[-1])

IF vers GE 5.2 THEN BEGIN
  print, 'Reader does not support version 5.2 HDF files'
  print, 'Exiting...'
  exit 

ENDIF ELSE IF vers EQ 5.1 THEN BEGIN
	;  Create the structure to read from the HDF file needed to dynamically size
	;  the data arrays
	solar_size = {event_id:0L, $
	                 Date:0L, $
	                 Year_Fraction:0., $
	                 Latitude:0., $
	                 Longitude:0., $
	                 Time:0L, $
	                 int_fill_value:0L, $
	                 flt_fill_value:0., $
	                 mission_id:0L, $
	                 L0DO_Version:0., $
	                 CCDVersion:0., $
	                 L0_Version:0., $
	                 Software_Version:0., $
	                 Dataproduct_Version:0., $
	                 Spectroscopic_DataBase_Version:0., $
	                 GRAM95_Version:0., $
	                 Met_Version:0., $
	                 bin_height:0., $
	                 num_bins:0L, $
	                 num_met_grid:0L, $
	                 num_aer_channels:0L,$
	                 num_grnd_trk:0L, $
	                 num_aer_bins:0L}
	!QUIET = 1

	;  Start to extract the needed information
	fileID = H5F_OPEN(fileName)

	;  Get the number of groups in the root group

	nMainGroups = h5g_get_nmembers(fileID,"/")

	;  Select the first group where some of this data resides
	g0Name = h5g_get_member_Name(fileID,"/",0)
	g0Path = "/" + g0Name
	g01Name = h5g_get_member_Name(fileID,g0Path,0)
	g01Path = g0Name + "/" + g01Name
	dataset_id = h5d_open(fileID,g01Path)
	data = h5d_read(dataset_id)
	solar_size.event_id = data.(0)
	solar_size.Year_Fraction= data.(1)
	solar_size.Date= data.(2)
	solar_size.Latitude = data.(3)
	solar_size.Longitude = data.(4)
	solar_size.Time = data.(5)
	solar_size.int_fill_value = data.(6)
	solar_size.flt_fill_value = data.(7)
	solar_size.mission_id = data.(8)
	h5d_close,dataset_id

	g1Name = h5g_get_member_Name(fileID,"/",1)
	g1Path = "/" + g1Name
	g11Name = h5g_get_member_Name(fileID,g1Path,0)
	g11Path = g1Name + "/" + g11Name
	dataset_id = h5d_open(fileID,g11Path)
	data = h5d_read(dataset_id)
	solar_size.L0DO_Version = data.(0)
	solar_size.CCDVERSION = data.(1)
	solar_size.L0_Version = data.(2)
	solar_size.Spectroscopic_DataBase_Version = data.(3)
	solar_size.GRAM95_Version = data.(4)
	solar_size.Met_Version = data.(5)
	solar_size.Software_Version = data.(6)
	solar_size.Dataproduct_Version = data.(7)
	h5d_close,dataset_id

	g2Name = h5g_get_member_Name(fileID,"/",2)
	g2Path = "/" + g2Name
	g21Name = h5g_get_member_Name(fileID,g2Path,0)
	g21Path = g2Path + "/" + g21Name
	dataset_id = h5d_open(fileID,g21Path)
	data = h5d_read(dataset_id)
	solar_size.bin_height = data.(0)
	solar_size.num_bins = data.(1)
	solar_size.num_met_grid = data.(2)
	solar_size.num_aer_channels = data.(3)
	solar_size.num_grnd_trk = data.(4)
	solar_size.num_aer_bins = data.(5)
	h5d_close, dataset_id

	;  Create the aerosol extinction channel data
	;  NOTE: The tag AerosolExt is an array containing this structure
	;        for each channel.  To reference the i-th element of the 
	;        AerosolExt, the correct syntax is solar.AerosolExt(i).aerext
	aerosol_ext = { aerosol_ext, $
	                aerext:fltarr(solar_size.num_aer_bins), $
	                aerext_uncert:fltarr(solar_size.num_aer_bins), $
	                aerQA:lonarr(solar_size.num_aer_bins)   }
		

	;  Create the data structure to be read in
	;  Define the solar structure.
	solar = { event_id:0L, $
	      Date:0L, $
	      Year_Fraction:0., $
	      Latitude:0., $
	      Longitude:0., $
	      Time:0L, $
	      int_fill_value:0L, $
	      flt_fill_value:0., $
	      mission_id:0L, $
	      L0DO_Version:0., $
	      CCDTable_Version:0L, $
	      L0_Version:0., $
	      Software_Version:0., $
	      Dataproduct_Version:0., $
	      Spectroscopic_Database_Version:0., $
	      GRAM95_Version:0., $
	      Met_Version:0., $
	      bin_height:0., $
	      num_bins:0L, $
	      num_met_grid:0L, $
	      num_aer_channels:0L,$
	      num_grnd_trk:0L, $
	      num_aer_bins:0L, $
	      sc_evt_type:0L, $
	      gnd_evt_type:0L, $
	      BetaAngle_Solar:0., $
	      Aurora_Flag:0L, $
	      Ephemeris_Source:0L, $
	      gt_date:lonarr(solar_size.num_grnd_trk), $
	      gt_time:lonarr(solar_size.num_grnd_trk), $
	      gt_latitude:fltarr(solar_size.num_grnd_trk), $
	      gt_longitude:fltarr(solar_size.num_grnd_trk), $
	      gt_ray_dir:fltarr(solar_size.num_grnd_trk), $
	      Space_Craft_Alt:fltarr(solar_size.num_grnd_trk), $
	      Space_Craft_Lat:fltarr(solar_size.num_grnd_trk), $
	      Space_Craft_Lon:fltarr(solar_size.num_grnd_trk), $
	      Homogeneity:lonarr(solar_size.num_bins), $
	      Altitude:fltarr(solar_size.num_bins), $
	      Geopotential_Alt:fltarr(solar_size.num_bins), $
	      Temperature:fltarr(solar_size.num_bins), $
	      Temperature_uncert:fltarr(solar_size.num_bins), $
	      Pressure:fltarr(solar_size.num_bins), $
	      Pressure_uncert:fltarr(solar_size.num_bins), $
	      Neutral_Density:fltarr(solar_size.num_bins), $
	      Neutral_Density_uncert:fltarr(solar_size.num_bins), $
	      Temp_Pressure_Source:lonarr(solar_size.num_bins), $
	      trop_temp:0., $
	      trop_alt:0., $
	      trop_press:0., $
	      met_pressure:fltarr(solar_size.num_met_grid), $
	      met_temp:fltarr(solar_size.num_met_grid), $
	      met_temp_unc:fltarr(solar_size.num_met_grid), $
	      met_altitude:fltarr(solar_size.num_met_grid), $
	      met_Source:0L, $
	      CCD_Temperature:0., $
	      Spectrometer_Zenith_Temperature:0., $
	      CCD_Temperature_minus_TEC:0., $
	      Ephemeris_Quality:0L, $
	      QAFlag:0L, $
	      QAFlag_Altitude:lonarr(solar_size.num_bins), $
	      SpecCalShift:0., $
	      SpecCalStretch:0., $
              Ozone_Composite:fltarr(solar_size.num_bins), $
	      Ozone_Composite_uncert:fltarr(solar_size.num_bins), $
	      Ozone_Composite_QA:lonarr(solar_size.num_bins), $
	      Ozone_Mes:fltarr(solar_size.num_bins), $
	      Ozone_Mes_uncert:fltarr(solar_size.num_bins), $
	      Ozone_Mes_QA:lonarr(solar_size.num_bins), $
	      Ozone_MLR:fltarr(solar_size.num_bins), $
	      Ozone_MLR_uncert:fltarr(solar_size.num_bins), $
	      Ozone_MLR_QA:lonarr(solar_size.num_bins), $
	      Ozone_aO3:fltarr(solar_size.num_bins), $
	      Ozone_aO3_uncert:fltarr(solar_size.num_bins), $
	      Ozone_aO3_QA:lonarr(solar_size.num_bins), $
	      H2O:fltarr(solar_size.num_bins), $
	      H2O_uncert:fltarr(solar_size.num_bins), $
	      H2O_QA:lonarr(solar_size.num_bins), $
	      NO2:fltarr(solar_size.num_bins), $
	      NO2_uncert:fltarr(solar_size.num_bins), $
	      NO2_QA:lonarr(solar_size.num_bins), $
	      RetTemp:fltarr(solar_size.num_bins), $
	      RetTemp_uncert:fltarr(solar_size.num_bins), $
	      RetPress:fltarr(solar_size.num_bins), $
	      RetPress_uncert:fltarr(solar_size.num_bins), $
	      RetTP_QA:lonarr(solar_size.num_bins), $
	      Aer_wavelength:fltarr(solar_size.num_aer_channels), $
	      Aer_width:fltarr(solar_size.num_aer_channels), $
	      Strat_Aer_OD:fltarr(solar_size.num_aer_channels), $
	      Strat_Aer_OD_uncert:fltarr(solar_size.num_aer_channels), $
	      Strat_Aer_OD_QA:lonarr(solar_size.num_aer_channels), $
	      AerosolExt:replicate(aerosol_ext, solar_size.num_aer_channels) , $
	      Molecular_SCT:fltarr(solar_size.num_aer_channels), $
	      Molecular_SCT_uncert:fltarr(solar_size.num_aer_channels)}

	;  Now we read in the data from the solar_size array
	solar.event_id = solar_size.event_id
	solar.Date = solar_size.Date
	solar.Year_Fraction = solar_size.Year_Fraction
	solar.Latitude = solar_size.Latitude
	solar.Longitude = solar_size.Longitude
	solar.Time = solar_size.Time
	solar.int_fill_value = solar_size.int_fill_value
	solar.flt_fill_value = solar_size.flt_fill_value
	solar.mission_id = solar_size.mission_id
	solar.L0DO_Version = solar_size.L0DO_Version
	solar.CCDTable_Version = solar_size.CCDVersion
	solar.L0_Version = solar_size.L0_Version
	solar.Software_Version = solar_size.Software_Version
	solar.Dataproduct_Version = solar_size.Dataproduct_Version
	solar.Spectroscopic_Database_Version = solar_size.Spectroscopic_Database_Version
	solar.GRAM95_Version = solar_size.GRAM95_Version
	solar.Met_Version = solar_size.Met_Version
	solar.bin_height = solar_size.bin_height
	solar.num_bins = solar_size.num_bins
        solar.num_met_grid = solar_size.num_met_grid
	solar.num_aer_channels = solar_size.num_aer_channels
	solar.num_grnd_trk = solar_size.num_grnd_trk
	solar.num_aer_bins = solar_size.num_aer_bins

	;  Now that the structure exists and has been filled with the previously
	;  read data, we now fill the rest of the structure
	g4Name = h5g_get_member_Name(fileID,"/",3)
	g4Path = "/" + g4Name
	dsName = h5g_get_member_Name(fileID,g4Path,0)
	dsPath = g4Path + "/" + dsName
	dataset_id = h5d_open(fileID,dsPath)
	data = h5d_read(dataset_id)
	solar.sc_evt_type = data.(0)
	solar.gnd_evt_type = data.(1)
	solar.BetaAngle_Solar = data.(2)
	solar.Ephemeris_Source = data.(4)
	solar.Aurora_Flag = data.(3)
	h5d_close,dataset_id

	g41Name = h5g_get_member_Name(fileID,g4Path,1)
	g41Path = g4Path + "/" + g41Name
	dsName = h5g_get_member_Name(fileID,g41Path,0)
	dsPath = g41Path + "/" + dsName
	dataset_id = h5d_open(fileID,dsPath)
	data = h5d_read(dataset_id)
	solar.gt_date = data.(0)
	solar.gt_time = data.(1)
	solar.gt_latitude = data.(2)
	solar.gt_longitude = data.(3)
	solar.gt_ray_dir = data.(4)
	h5d_close,dataset_id

	g42Name = h5g_get_member_Name(fileID,g4Path,2)
	g42Path = g4Path + "/" + g42Name
	dsName = h5g_get_member_Name(fileID,g42Path,0)
	dsPath = g42Path + "/" + dsName
	dataset_id = h5d_open(fileID,dsPath)
	data = h5d_read(dataset_id)
	solar.Space_Craft_Lat = data.(0)
	solar.Space_Craft_Lon = data.(1)
	solar.Space_Craft_Alt= data.(2)
	h5d_close,dataset_id

	g43Name = h5g_get_member_name(fileID, g4path,3)
	g43Path = g4Path + "/" + g43Name 
	dsName = h5g_get_member_Name(fileID, g43Path,0)
	dsPath = g43Path + "/" + dsName 
	dataset_id = h5d_open(fileID, dsPath)
	data = h5d_read(dataset_id)
	solar.CCD_Temperature = data.(0)
        solar.Spectrometer_Zenith_Temperature = data.(1)
        solar.CCD_Temperature_minus_TEC = data.(2)
        solar.Ephemeris_Quality = data.(3)
        solar.SpecCalShift = data.(4)
        solar.SpecCalStretch = data.(5)
	solar.QAFlag = data.(6)
	h5d_close, dataset_id

	g44Name = h5g_get_member_name(fileID, g4path,4)
        g44Path = g4Path + "/" + g44Name
        dsName = h5g_get_member_Name(fileID, g44Path,0)
        dsPath = g44Path + "/" + dsName
        dataset_id = h5d_open(fileID, dsPath)
        data = h5d_read(dataset_id)
        solar.QAFlag_Altitude = data.(0)
        h5d_close, dataset_id


	;  Now we move onto Section 5 - Altitude based data
	g5Name = h5g_get_member_Name(fileID,"/",4)
	g5Path = "/" + g5Name

	;Will need to be moved and fields added
        dsName = h5g_get_member_Name(fileID,g5Path,0)
        dsPath = g5Path + "/" + dsName
        dataset_id = h5d_open(fileID,dsPath)
        data = h5d_read(dataset_id)
        solar.Homogeneity = data.(0)
        solar.Altitude = data.(1)
        solar.Geopotential_Alt = data.(2)
        h5d_close,dataset_id
	
	g51Name = h5g_get_member_Name(fileID,g5Path,1)
	g51Path = g5Path + "/" + g51Name
	dsName = h5g_get_member_Name(fileID,g51Path,0)
	dsPath = g51Path + "/" + dsName
	dataset_id = h5d_open(fileID,dsPath)
	data = h5d_read(dataset_id)
	solar.Temperature = data.(0)
	solar.Temperature_uncert = data.(1)
	solar.Pressure = data.(2)
	solar.Pressure_uncert = data.(3)
	solar.Neutral_Density = data.(4)
	solar.Neutral_Density_uncert = data.(5)
        solar.Temp_Pressure_Source = data.(6)
	h5d_close,dataset_id

	g51aName = h5g_get_member_Name(fileID,g5Path,2)
	g51aPath = g5Path + "/" + g51aName
	dsName = h5g_get_member_Name(fileID,g51aPath,0)
	dsPath = g51aPath + "/" + dsName
	dataset_id = h5d_open(fileID,dsPath)
	data = h5d_read(dataset_id)
	solar.trop_alt = data.(0)
	solar.trop_temp = data.(1)
	solar.trop_press = data.(2)
        solar.met_source = data.(3)
	h5d_close,dataset_id

	g51bName = h5g_get_member_Name(fileID,g5Path,3)
        g51bPath = g5Path + "/" + g51bName
        dsName = h5g_get_member_Name(fileID,g51bPath,0)
        dsPath = g51bPath + "/" + dsName
        dataset_id = h5d_open(fileID,dsPath)
        data = h5d_read(dataset_id)
	solar.met_altitude = data.(0)
        solar.met_temp = data.(1)
        solar.met_temp_unc = data.(2)
        solar.met_pressure = data.(3)
        h5d_close,dataset_id

	g52Name = h5g_get_member_Name(fileID,g5Path,4)
	g52Path = g5Path + "/" + g52Name
	dsName = h5g_get_member_Name(fileID,g52Path,0)
	dsPath = g52Path + "/" + dsName
	dataset_id = h5d_open(fileID,dsPath)
	data = h5d_read(dataset_id)
	solar.Ozone_Composite = data.(0)
	solar.Ozone_Composite_uncert = data.(1)
	solar.Ozone_Composite_QA = data.(2)
	h5d_close,dataset_id

	g52aName = h5g_get_member_Name(fileID,g5Path,5)
	g52aPath = g5Path + "/" + g52aName
	dsName = h5g_get_member_Name(fileID,g52aPath,0)
	dsPath = g52aPath + "/" + dsName
	dataset_id = h5d_open(fileID,dsPath)
	data = h5d_read(dataset_id)
	solar.Ozone_Mes = data.(0)
	solar.Ozone_Mes_uncert = data.(1)
	solar.Ozone_Mes_QA = data.(2)
	h5d_close,dataset_id

	g52bName = h5g_get_member_Name(fileID,g5Path,6)
	g52bPath = g5Path + "/" + g52bName
	dsName = h5g_get_member_Name(fileID,g52bPath,0)
	dsPath = g52bPath + "/" + dsName
	dataset_id = h5d_open(fileID,dsPath)
	data = h5d_read(dataset_id)
	solar.Ozone_MLR = data.(0)
	solar.Ozone_MLR_uncert = data.(1)
	solar.Ozone_MLR_QA = data.(2)
	h5d_close,dataset_id

	g52cName = h5g_get_member_Name(fileID,g5Path,7)
	g52cPath = g5Path + "/" + g52cName
	dsName = h5g_get_member_Name(fileID,g52cPath,0)
	dsPath = g52cPath + "/" + dsName
	dataset_id = h5d_open(fileID,dsPath)
	data = h5d_read(dataset_id)
	solar.Ozone_ao3 = data.(0)
	solar.Ozone_ao3_uncert = data.(1)
	solar.Ozone_aO3_QA = data.(2)
	h5d_close,dataset_id

	g53Name = h5g_get_member_Name(fileID,g5Path,8)
	g53Path = g5Path + "/" + g53Name
	dsName = h5g_get_member_Name(fileID,g53Path,0)
	dsPath = g53Path + "/" + dsName
	dataset_id = h5d_open(fileID,dsPath)
	data = h5d_read(dataset_id)
	solar.H2O = data.(0)
	solar.H2O_uncert = data.(1)
	solar.H2O_QA = data.(2)
	h5d_close,dataset_id

	g54Name = h5g_get_member_Name(fileID,g5Path,9)
	g54Path = g5Path + "/" + g54Name
	dsName = h5g_get_member_Name(fileID,g54Path,0)
	dsPath = g54Path + "/" + dsName
	dataset_id = h5d_open(fileID,dsPath)
	data = h5d_read(dataset_id)
	solar.NO2 = data.(0)
	solar.NO2_uncert = data.(1)
	solar.NO2_QA = data.(2)
	h5d_close,dataset_id

	g55Name = h5g_get_member_Name(fileID,g5Path,10)
	g55Path = g5Path + "/" + g55Name
	dsName = h5g_get_member_Name(fileID,g55Path,0)
	dsPath = g55Path + "/" + dsName
	dataset_id = h5d_open(fileID,dsPath)
	data = h5d_read(dataset_id)
	solar.RetTemp = data.(0)
	solar.RetTemp_uncert = data.(1)
	solar.RetPress = data.(2)
	solar.RetPress_uncert = data.(3)
	solar.RetTP_QA = data.(4)
	h5d_close,dataset_id

	; Now we move on to section 6, 6.1 and 6.2
        g6Name = h5g_get_member_Name(fileID,"/",5)
	g6Path = "/" + g6Name
	dsName = h5g_get_member_Name(fileID,g6Path,0)
	dsPath = g6Path + "/" + dsName
	dataset_id = h5d_open(fileID,dsPath)
	data = h5d_read(dataset_id)
	solar.Aer_wavelength = data.(0)
	solar.Aer_width = data.(1)
        solar.Molecular_SCT = data.(2)
        solar.Molecular_SCT_uncert = data.(3)
	solar.Strat_Aer_OD = data.(4)
	solar.Strat_Aer_OD_uncert = data.(5)
	solar.Strat_Aer_OD_QA = data.(6)
	h5d_close,dataset_id

	g61Name = h5g_get_member_Name(fileID,"/",6)
        g61Path =  "/" + g61Name
        dsName = h5g_get_member_Name(fileID,g61Path,0)
        dsPath = g61Path + "/" + dsName
        dataset_id = h5d_open(fileID,dsPath)
        data = h5d_read(dataset_id)
	for k = 0L,solar.num_aer_channels-1 do begin
	   solar.AerosolExt(k).aerext = data.(3*k)
	   solar.AerosolExt(k).aerext_uncert = data.(3*k+1)
	   solar.AerosolExt(k).aerQA = data.(3*k+2)
	endfor
	h5d_close,dataset_id

	;Close file
	H5F_CLOSE, fileID
	H5_CLOSE 

        ;Decode Bits 
        ;Add Separate Flags for Event QA BitFlag Entries, IDL 1/0 = True/False
        solar_addtags={HexErrFlag:0L,ContWindowClosedFlag:0L,TimeQualFlag:0L,DMPExoFlag:0L,BlockExoFlag:0L,$
           SpectralCalFlag:0L,SolarEclipseFlag:0L,DMPAltFlag:lonarr(solar_size.num_bins)}

        solar=create_struct(solar,solar_addtags)

        if ((solar.QAFlag and 2L^0) ne 0) then solar.HexErrFlag=1 else solar.HexErrFlag=0
        if ((solar.QAFlag and 2L^1) ne 0) then solar.ContWindowClosedFlag=1 else solar.ContWindowClosedFlag=0
        if ((solar.QAFlag and 2L^2) ne 0) then solar.TimeQualFlag=1 else solar.TimeQualFlag=0
        if ((solar.QAFlag and 2L^3) ne 0) then solar.DMPExoFlag=1 else solar.DMPExoFlag=0
        if ((solar.QaFlag and 2L^4) ne 0) then solar.BlockExoFlag=1 else solar.BlockExoFlag=0
        if ((solar.QAFlag and 2L^5) ne 0) then solar.SpectralCalFlag=1 else solar.SpectralCalFlag=0
        if ((solar.QAFlag and 2L^6) ne 0) then solar.SolarEclipseFlag=1 else solar.SolarEclipseFlag=0

        ;Altitude Based Bit Flags
        for i = 0,solar_size.num_bins-1 do begin
          if ((solar.QAFlag_Altitude(i) and 2L^0) ne 0) then solar.DMPAltFlag(i)=1 else solar.DMPAltFlag(i)=0
        endfor

ENDIF ELSE BEGIN
	print, 'HDF5 files only available for v5.1 and later'
	status = 'ERROR'
ENDELSE

end
