pro read_solar_trans_hdf5, fileName, status, solar_trans

;Disclaimers
;No Warranty: THE SUBJECT SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY OF
;ANY KIND, EITHER EXPRESSED, IMPLIED, OR STATUTORY, INCLUDING, BUT NOT LIMITED
;TO, ANY WARRANTY THAT THE SUBJECT SOFTWARE WILL CONFORM TO SPECIFICATIONS,
;ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
;OR FREEDOM FROM INFRINGEMENT, ANY WARRANTY THAT THE SUBJECT SOFTWARE WILL BE
;ERROR FREE, OR ANY WARRANTY THAT DOCUMENTATION, IF PROVIDED, WILL CONFORM TO
;THE SUBJECT SOFTWARE. THIS AGREEMENT DOES NOT, IN ANY MANNER, CONSTITUTE AN
;ENDORSEMENT BY GOVERNMENT AGENCY OR ANY PRIOR RECIPIENT OF ANY RESULTS,
;RESULTING DESIGNS, HARDWARE, SOFTWARE PRODUCTS OR ANY OTHER APPLICATIONS
;RESULTING FROM USE OF THE SUBJECT SOFTWARE.  FURTHER, GOVERNMENT AGENCY
;DISCLAIMS ALL WARRANTIES AND LIABILITIES REGARDING THIRD-PARTY SOFTWARE, IF
;PRESENT IN THE ORIGINAL SOFTWARE, AND DISTRIBUTES IT "AS IS." 
 
;Waiver and Indemnity:  RECIPIENT AGREES TO WAIVE ANY AND ALL CLAIMS AGAINST
;THE UNITED STATES GOVERNMENT, ITS CONTRACTORS AND SUBCONTRACTORS, AS WELL AS
;ANY PRIOR RECIPIENT.  IF RECIPIENT'S USE OF THE SUBJECT SOFTWARE RESULTS IN
;ANY LIABILITIES, DEMANDS, DAMAGES, EXPENSES OR LOSSES ARISING FROM SUCH USE,
;INCLUDING ANY DAMAGES FROM PRODUCTS BASED ON, OR RESULTING FROM, RECIPIENT'S
;USE OF THE SUBJECT SOFTWARE, RECIPIENT SHALL INDEMNIFY AND HOLD HARMLESS THE
;UNITED STATES GOVERNMENT, ITS CONTRACTORS AND SUBCONTRACTORS, AS WELL AS ANY
;PRIOR RECIPIENT, TO THE EXTENT PERMITTED BY LAW.  RECIPIENT'S SOLE REMEDY FOR
;ANY SUCH MATTER SHALL BE THE IMMEDIATE, UNILATERAL TERMINATION OF THIS
;AGREEMENT.

;  Purpose:
;  This provides the data structures required to read in the solar SAGE III
;  event species data from HDF version 5 files
;
;  Author:
;  Marsha LaRosee, SSAI, marsha.a.larosee@nasa.gov
;  Carter Hulsey
;  Michael Heitz
;  David Huber
;  Robbie Manion
;
;  Input Variables:
;  fileName - Name of the HDF5 file to be read in (g3b.ssp.*)
;
;  Output Variables:
;  status - Status variable of the read

status = 'OK'

;Find Version from file name and if greater than or equal to 5.1
;Use the updated Reader
splt_file = STRSPLIT(filename,'v',/EXTRACT)
vers = FLOAT(splt_file[-1])

IF vers GE 5.2 THEN BEGIN
  print, 'Reader does not support version 5.2 HDF files'
  print, 'Exiting...'
  exit

ENDIF ELSE IF vers EQ 5.1 THEN BEGIN

;  Create the structure for read from binary file of the data needed
;  to dynamically size data arrays.

	solar_trans_size = {solar_trans_size, event_id:0L, $
	                 Date:0L, $
	                 fraction_time:0., $
	                 Latitude:0., $
	       	         Longitude:0., $
	                 Time:0L, $
	                 int_fill_value:0L, $
	                 flt_fill_value:0., $
	                 mission_id:0L, $
	                 L0DO_Version:0., $
	                 CCDVersion:0., $
	                 L0_Version:0., $
	                 Software_Version:0., $
	                 Dataproduct_Version:0., $
	                 Spectroscopic_DataBase_Version:0., $
	                 GRAM95_Version:0., $
	                 Met_Version:0., $
	                 bin_height:0., $
	                 profile_count:0L, $
	                 num_grnd_trk:0L, $
	                 num_press_grid:0L, $
	                 num_CCDPxlGrps:0L, $
	                 num_alt_bins:0L }
	!QUIET = 1

	;  Start to extract the needed information

        fileID = H5F_OPEN(fileName)

        ;  Get the number of groups in the root group

        nMainGroups = h5g_get_nmembers(fileID,"/")

        ;  Select the first group where some of this data resides
        g0Name = h5g_get_member_Name(fileID,"/",0)
        g0Path = "/" + g0Name
        g01Name = h5g_get_member_Name(fileID,g0Path,0)
        g01Path = g0Name + "/" + g01Name
        dataset_id = h5d_open(fileID,g01Path)
        data = h5d_read(dataset_id)
        solar_trans_size.event_id = data.(0)
        solar_trans_size.fraction_time = data.(1)
        solar_trans_size.Date = data.(2)
	solar_trans_size.Latitude = data.(3)
	solar_trans_size.Longitude = data.(4)
	solar_trans_size.Time = data.(5)
        solar_trans_size.int_fill_value = data.(6)
        solar_trans_size.flt_fill_value = data.(7)
        solar_trans_size.mission_id = data.(8)
        h5d_close,dataset_id

        g1Name = h5g_get_member_Name(fileID,"/",1)
        g1Path = "/" + g1Name
        g11Name = h5g_get_member_Name(fileID,g1Path,0)
        g11Path = g1Name + "/" + g11Name
        dataset_id = h5d_open(fileID,g11Path)
        data = h5d_read(dataset_id)
        solar_trans_size.L0DO_Version = data.(0)
	solar_trans_size.CCDVersion = data.(1)
        solar_trans_size.L0_Version = data.(2)
	solar_trans_size.Spectroscopic_DataBase_Version = data.(3)
        solar_trans_size.GRAM95_Version = data.(4)
        solar_trans_size.Met_Version = data.(5)
        solar_trans_size.Software_Version = data.(6)
        solar_trans_size.Dataproduct_Version = data.(7)
        h5d_close,dataset_id

        g2Name = h5g_get_member_Name(fileID,"/",2)
        g2Path = "/" + g2Name
        g21Name = h5g_get_member_Name(fileID,g2Path,0)
        g21Path = g2Path + "/" + g21Name
        dataset_id = h5d_open(fileID,g21Path)
        data = h5d_read(dataset_id)
        solar_trans_size.bin_height = data.(0)
        solar_trans_size.profile_count = data.(1)
        solar_trans_size.num_grnd_trk = data.(2)
        solar_trans_size.num_press_grid = data.(3)
        solar_trans_size.num_CCDPxlGrps = data.(4)
        solar_trans_size.num_alt_bins = data.(5)



        ;  Create a structure for the pixel group transmission profile.
        ;  NOTE: The tag pg_trans_profiles is an array containing this structure
        ;        for each channel.  To reference the i-th element of the array,
        ;        the correct syntax is solar.pg_trans_profile(i).Transmission

        pg_trans_profile = { pg_trans_profile, $
                      Transmission:fltarr(solar_trans_size.num_alt_bins,2), $
                      TransQA:lonarr(solar_trans_size.num_alt_bins) }

	;Create the structure for read of all data from hdf5 file. 
	;  NOTE:  Last tag in the structure is an array containing the 
	;   pg_trans_profile structure.

	solar_trans = { event_id:0L, $
	      Date:0L, $
	      Year_Fraction:0., $
	      Latitude:0., $
	      Longitude:0., $
	      Time:0L, $
	      int_fill_value:0L, $
	      flt_fill_value:0., $
	      mission_id:0L, $
	      L0DO_Version:0., $
	      CCDVersion:0L, $
	      L0_Version:0., $
	      Software_Version:0., $
	      Dataproduct_Version:0., $
	      Spectroscopic_DataBase_Version:0., $
	      GRAM95_Version:0., $
	      Met_Version:0., $
	      bin_height:0., $
	      profile_count:0L, $
	      num_grnd_trk:0L, $
	      num_press_grid:0L, $
	      num_CCDPxlGrps:0L, $
	      num_alt_bins:0L, $
	      sc_evt_type:0L, $
	      gnd_evt_type:0L, $
	      BetaAngle_Solar:0., $
	      Aurora_Flag:0L, $
	      Ephemeris_Source:0L, $
	      gt_date:lonarr(solar_trans_size.num_grnd_trk), $
	      gt_time:lonarr(solar_trans_size.num_grnd_trk), $
	      gt_latitude:fltarr(solar_trans_size.num_grnd_trk), $
	      gt_longitude:fltarr(solar_trans_size.num_grnd_trk), $
	      gt_ray_dir:fltarr(solar_trans_size.num_grnd_trk), $
	      Space_Craft_Lat:fltarr(solar_trans_size.num_grnd_trk), $
	      Space_Craft_Lon:fltarr(solar_trans_size.num_grnd_trk), $
	      Space_Craft_Alt:fltarr(solar_trans_size.num_grnd_trk), $
	      Altitude:fltarr(solar_trans_size.num_alt_bins), $
	      Geopotential_Alt:fltarr(solar_trans_size.num_alt_bins), $
	      Temperature:fltarr(solar_trans_size.num_alt_bins), $
	      Temperature_uncert:fltarr(solar_trans_size.num_alt_bins), $
	      Pressure:fltarr(solar_trans_size.num_alt_bins), $
	      Pressure_uncert:fltarr(solar_trans_size.num_alt_bins), $
	      Neutral_Density:fltarr(solar_trans_size.num_alt_bins), $
	      Neutral_Density_uncert:fltarr(solar_trans_size.num_alt_bins), $
	      Temp_Pressure_Source:lonarr(solar_trans_size.num_alt_bins), $
	      trop_temp:0., $
	      trop_alt:0., $
	      trop_press:0., $
	      met_pressure:fltarr(solar_trans_size.num_press_grid), $
	      met_temp:fltarr(solar_trans_size.num_press_grid), $
	      met_temp_unc:fltarr(solar_trans_size.num_press_grid), $
	      met_altitude:fltarr(solar_trans_size.num_press_grid), $
	      met_Source:0L, $
	      CCD_Temperature:0., $
	      Spectrometer_Zenith_Temperature:0., $
	      CCD_Temperature_minus_TEC:0., $
	      Ephemeris_Quality:0L, $
	      QAFlag:0L, $
	      QAFlag_Altitude:lonarr(solar_trans_size.num_alt_bins), $
	      SpecCalStretch:0.,$
	      SpecCalShift:0.,$
	      start_pixel_num:lonarr(solar_trans_size.num_CCDPxlGrps), $
	      end_pixel_num:lonarr(solar_trans_size.num_CCDPxlGrps), $
	      central_wavelength:fltarr(solar_trans_size.num_CCDPxlGrps), $
	      half_bandwidth:fltarr(solar_trans_size.num_CCDPxlGrps), $
	      pg_trans_profiles:replicate( pg_trans_profile , 87 ) }


 	;  Now we read in the data from the solar_size array
        solar_trans.event_id = solar_trans_size.event_id
	solar_trans.Date = solar_trans_size.Date
	solar_trans.Year_Fraction = solar_trans_size.fraction_time
	solar_trans.Latitude = solar_trans_size.Latitude 
	solar_trans.Longitude = solar_trans_size.Longitude
        solar_trans.Time = solar_trans_size.Time
        solar_trans.int_fill_value = solar_trans_size.int_fill_value
        solar_trans.flt_fill_value = solar_trans_size.flt_fill_value
        solar_trans.mission_id = solar_trans_size.mission_id
        solar_trans.L0DO_Version = solar_trans_size.L0DO_Version
	solar_trans.CCDVersion = solar_trans_size.CCDVersion 
        solar_trans.L0_Version = solar_trans_size.L0_Version
        solar_trans.Software_Version = solar_trans_size.Software_Version
        solar_trans.Dataproduct_Version = solar_trans_size.Dataproduct_Version
        solar_trans.Spectroscopic_DataBase_Version = solar_trans_size.Spectroscopic_DataBase_Version
	solar_trans.GRAM95_Version = solar_trans_size.GRAM95_Version
        solar_trans.Met_Version = solar_trans_size.Met_Version
        solar_trans.bin_height = solar_trans_size.bin_height
        solar_trans.profile_count = solar_trans_size.profile_count
        solar_trans.num_grnd_trk = solar_trans_size.num_grnd_trk
        solar_trans.num_press_grid = solar_trans_size.num_press_grid
        solar_trans.num_CCDPxlGrps = solar_trans_size.num_CCDPxlGrps
        solar_trans.num_alt_bins = solar_trans_size.num_alt_bins

	;  Now that the structure exists and has been filled with the previously
        ;  read data, we now fill the rest of the structure
        g4Name = h5g_get_member_Name(fileID,"/",3)
        g4Path = "/" + g4Name
        dsName = h5g_get_member_Name(fileID,g4Path,0)
        dsPath = g4Path + "/" + dsName
        dataset_id = h5d_open(fileID,dsPath)
        data = h5d_read(dataset_id)
        solar_trans.sc_evt_type = data.(0)
        solar_trans.gnd_evt_type = data.(1)
        solar_trans.BetaAngle_Solar = data.(2)
        solar_trans.Ephemeris_Source = data.(4)
        solar_trans.Aurora_Flag = data.(3)
	h5d_close,dataset_id

	g41Name = h5g_get_member_Name(fileID,g4Path,1)
        g41Path = g4Path + "/" + g41Name
        dsName = h5g_get_member_Name(fileID,g41Path,0)
        dsPath = g41Path + "/" + dsName
        dataset_id = h5d_open(fileID,dsPath)
        data = h5d_read(dataset_id)
        solar_trans.gt_date = data.(0)
        solar_trans.gt_time = data.(1)
        solar_trans.gt_latitude = data.(2)
        solar_trans.gt_longitude = data.(3)
        solar_trans.gt_ray_dir = data.(4)
        h5d_close,dataset_id

        g42Name = h5g_get_member_Name(fileID,g4Path,2)
        g42Path = g4Path + "/" + g42Name
        dsName = h5g_get_member_Name(fileID,g42Path,0)
        dsPath = g42Path + "/" + dsName
        dataset_id = h5d_open(fileID,dsPath)
        data = h5d_read(dataset_id)
        solar_trans.Space_Craft_Lat = data.(0)
        solar_trans.Space_Craft_Lon = data.(1)
        solar_trans.Space_Craft_Alt = data.(2)
        h5d_close,dataset_id

	g43Name = h5g_get_member_name(fileID, g4path,3)
        g43Path = g4Path + "/" + g43Name
        dsName = h5g_get_member_Name(fileID, g43Path,0)
        dsPath = g43Path + "/" + dsName
        dataset_id = h5d_open(fileID, dsPath)
        data = h5d_read(dataset_id)
        solar_trans.CCD_Temperature = data.(0)
        solar_trans.Spectrometer_Zenith_Temperature = data.(1)
        solar_trans.CCD_Temperature_minus_TEC = data.(2)
        solar_trans.Ephemeris_Quality = data.(3)
        solar_trans.SpecCalShift = data.(4)
        solar_trans.SpecCalStretch = data.(5)
        solar_trans.QAFlag = data.(6)
        h5d_close, dataset_id

        g44Name = h5g_get_member_name(fileID, g4path,4)
        g44Path = g4Path + "/" + g44Name
        dsName = h5g_get_member_Name(fileID, g44Path,0)
        dsPath = g44Path + "/" + dsName
        dataset_id = h5d_open(fileID, dsPath)
        data = h5d_read(dataset_id)
        solar_trans.QAFlag_Altitude = data.(0)
        h5d_close, dataset_id


	;  Now we move onto Section 5 - Pressure Surface  based data
        g5Name = h5g_get_member_Name(fileID,"/",4)
        g5Path = "/" + g5Name

        dsName = h5g_get_member_Name(fileID,g5Path,0)
        dsPath = g5Path + "/" + dsName
        dataset_id = h5d_open(fileID,dsPath)
        data = h5d_read(dataset_id)
        solar_trans.met_altitude = data.(0)
        solar_trans.met_temp = data.(1)
        solar_trans.met_temp_unc = data.(2)
        solar_trans.met_pressure = data.(3)
        h5d_close,dataset_id

        dsName = h5g_get_member_Name(fileID,g5Path,1)
        dsPath = g5Path + "/" + dsName
        dataset_id = h5d_open(fileID,dsPath)
        data = h5d_read(dataset_id)
        solar_trans.trop_alt = data.(0)
        solar_trans.trop_temp = data.(1)
        solar_trans.trop_press = data.(2)
        solar_trans.met_source = data.(3)
        h5d_close,dataset_id

	; Section 6 CCD Information
        g6Name = h5g_get_member_Name(fileID,"/",5)
        g60Path = "/" + g6Name
        dsName = h5g_get_member_Name(fileID,g60Path,0)
        dsPath = g60Path + "/" + dsName
        dataset_id = h5d_open(fileID,dsPath)
        data = h5d_read(dataset_id)
        solar_trans.start_pixel_num = data.(0)
        solar_trans.end_pixel_num = data.(1)
        solar_trans.central_wavelength = data.(2)
        solar_trans.half_bandwidth = data.(3)
        h5d_close,dataset_id

	;Section 7 Altitude Based Profiles 
        g7Name = h5g_get_member_Name(fileID,"/",6)
        g7Path = "/" + g7Name

        dsName = h5g_get_member_Name(fileID,g7Path,0)
        dsPath = g7Path + "/" + dsName
        dataset_id = h5d_open(fileID,dsPath)
        data = h5d_read(dataset_id)
        solar_trans.Altitude = data.(0)
        solar_trans.Geopotential_Alt = data.(1)
	solar_trans.Pressure = data.(2)
	solar_trans.Pressure_uncert = data.(3)
        solar_trans.Temperature = data.(4)
        solar_trans.Temperature_uncert = data.(5)
        solar_trans.Neutral_Density = data.(6)
	solar_trans.Neutral_Density_uncert = data.(7) 
	solar_trans.Temp_Pressure_Source = data.(8)
        h5d_close,dataset_id

        for k = 0L,solar_trans_size.profile_count-1 DO BEGIN
                g71Name = h5g_get_member_Name(fileID,g7Path,k+1)
                g71Path = g7Path + "/" + g71Name
                dsName = h5g_get_member_Name(fileID,g71Path,0)
                dsPath = g71Path + "/" + dsName
                dataset_id = h5d_open(fileID,dsPath)
                data = h5d_read(dataset_id)
                solar_trans.pg_trans_profiles(k).Transmission(*,0) = data.(0)
                solar_trans.pg_trans_profiles(k).Transmission(*,1) = data.(1)
                solar_trans.pg_trans_profiles(k).TransQA = data.(2)
                h5d_close,dataset_id
        endfor
        h5f_close,fileID
	H5_CLOSE

        ;Add Separate Flags for Event QA BitFlag Entries, IDL 1/0 = True/False
        solar_addtags={HexErrFlag:0L,ContWindowClosedFlag:0L,TimeQualFlag:0L,DMPExoFlag:0L,BlockExoFlag:0L,$
           SpectralCalFlag:0L,SolarEclipseFlag:0L,DMPAltFlag:lonarr(solar_trans_size.num_alt_bins)}

        solar_trans=create_struct(solar_trans,solar_addtags)

        if ((solar_trans.QAFlag and 2L^0) ne 0) then solar_trans.HexErrFlag=1 else solar_trans.HexErrFlag=0
        if ((solar_trans.QAFlag and 2L^1) ne 0) then solar_trans.ContWindowClosedFlag=1 else solar_trans.ContWindowClosedFlag=0
        if ((solar_trans.QAFlag and 2L^2) ne 0) then solar_trans.TimeQualFlag=1 else solar_trans.TimeQualFlag=0
        if ((solar_trans.QAFlag and 2L^3) ne 0) then solar_trans.DMPExoFlag=1 else solar_trans.DMPExoFlag=0
        if ((solar_trans.QaFlag and 2L^4) ne 0) then solar_trans.BlockExoFlag=1 else solar_trans.BlockExoFlag=0
        if ((solar_trans.QAFlag and 2L^5) ne 0) then solar_trans.SpectralCalFlag=1 else solar_trans.SpectralCalFlag=0
        if ((solar_trans.QAFlag and 2L^6) ne 0) then solar_trans.SolarEclipseFlag=1 else solar_trans.SolarEclipseFlag=0

        ;Altitude Based Bit Flags
        for i = 0,solar_trans_size.num_alt_bins-1 do begin
          if ((solar_trans.QAFlag_Altitude(i) and 2L^0) ne 0) then solar_trans.DMPAltFlag(i)=1 else solar_trans.DMPAltFlag(i)=0
        endfor


ENDIF ELSE BEGIN
	status = 'ERROR'
	print, 'HDF5 files only available for v5.1 and later'
ENDELSE

end
